# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..types.event_request_user import EventRequestUser
from ..core.request_options import RequestOptions
from ..types.achievement_completion_response import AchievementCompletionResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.pydantic_utilities import parse_obj_as
from ..errors.unauthorized_error import UnauthorizedError
from ..types.error_body import ErrorBody
from ..errors.not_found_error import NotFoundError
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AchievementsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def complete(
        self,
        key: str,
        *,
        user: EventRequestUser,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AchievementCompletionResponse:
        """
        Mark an achievement as completed for a user.

        Parameters
        ----------
        key : str
            Unique reference of the achievement as set when created.

        user : EventRequestUser
            The user that completed the achievement.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AchievementCompletionResponse
            Successful operation

        Examples
        --------
        from trophy import EventRequestUser, TrophyApi

        client = TrophyApi(
            api_key="YOUR_API_KEY",
        )
        client.achievements.complete(
            key="finish-onboarding",
            user=EventRequestUser(
                id="18",
                email="jk.rowling@harrypotter.com",
                tz="Europe/London",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"achievements/{jsonable_encoder(key)}/complete",
            method="POST",
            json={
                "user": convert_and_respect_annotation_metadata(
                    object_=user, annotation=EventRequestUser, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    AchievementCompletionResponse,
                    parse_obj_as(
                        type_=AchievementCompletionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        ErrorBody,
                        parse_obj_as(
                            type_=ErrorBody,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorBody,
                        parse_obj_as(
                            type_=ErrorBody,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        ErrorBody,
                        parse_obj_as(
                            type_=ErrorBody,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncAchievementsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def complete(
        self,
        key: str,
        *,
        user: EventRequestUser,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AchievementCompletionResponse:
        """
        Mark an achievement as completed for a user.

        Parameters
        ----------
        key : str
            Unique reference of the achievement as set when created.

        user : EventRequestUser
            The user that completed the achievement.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AchievementCompletionResponse
            Successful operation

        Examples
        --------
        import asyncio

        from trophy import AsyncTrophyApi, EventRequestUser

        client = AsyncTrophyApi(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.achievements.complete(
                key="finish-onboarding",
                user=EventRequestUser(
                    id="18",
                    email="jk.rowling@harrypotter.com",
                    tz="Europe/London",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"achievements/{jsonable_encoder(key)}/complete",
            method="POST",
            json={
                "user": convert_and_respect_annotation_metadata(
                    object_=user, annotation=EventRequestUser, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    AchievementCompletionResponse,
                    parse_obj_as(
                        type_=AchievementCompletionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        ErrorBody,
                        parse_obj_as(
                            type_=ErrorBody,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorBody,
                        parse_obj_as(
                            type_=ErrorBody,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        ErrorBody,
                        parse_obj_as(
                            type_=ErrorBody,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
